#!/usr/bin/envpython
# -*-coding:UTF-8-*-
'''
@File    :   base.py
@Contact :   308711822@qq.com
@License :   (C) Copyright 2021-2225, Personal exclusive right.

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2023/6/16 22:40   小钟同学      1.0         None
'''

# 定义策略接口或抽象基类
from abc import ABC, abstractmethod
from agoku.libs.events.event_type import EventType


# 封装事件信息的类
class Event:

    def __init__(self, event_type: EventType = EventType.ERROR_LOGIN, data=None):
        self.event_type: EventType = event_type  # 事件类型 1: 逻辑错误，2：程序错误
        self.data = data  # 相关数据

class Strategy(ABC):
    event: Event
    def execute(self, notification_service,event: Event):
        '''
        该函数在发送事件的时候会进行触发调用
        :param event:
        :return:
        '''
        self.event = event
        self.notification_service=notification_service
        self.processor(event)  # 调用受子类实现的内部逻辑

    @abstractmethod
    def processor(self, event: Event):
        pass



#!/usr/bin/env
# -*-coding:UTF-8-*-
'''
@File    :   base.py
@Contact :   308711822@qq.com
@License :   (C) Copyright 2021-2225, Personal exclusive right.

@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2023/6/16 22:40   小钟同学      1.0         None
'''

from enum import Enum


class SubjectEventType(Enum):
    # 程序错误异常类型
    NOTIFICATION_ERROR = 'notification_error'
    # 通知回调异常类型
    CALLBACK_FOUND = 'callback_found'
    # 后天任务异步请求处理类型
    ASYNC_REQUEST = 'async_request'
    # 通知咪咕渠道触发了请求处理的异常
    ERROR_MIGU_NAIVE_ERROR = 'error_for_migu_native'
    # 通知咪咕的破解渠道发生异常错误的信息
    ERROR_MIGU_CRACK = 'error_for_migu_crack'
    # 咪咕号码按时间同步订购状态任务执行，因为咪咕的特殊性，我们需要使用同步机制处理
    MIGU_MM_ORDER_SYNC = 'migu_mm_order_sync'
    # 咪咕活动开通检测告警事件
    MIGU_MM_CK_ORDER_ELARM = 'migu_mm_ck_order_alarm'
    # 订单回调通知合作渠道
    CallbackNotificationChannel = 'callback_notification_channel'
    # 订单提交通知合作渠道
    SubmitNotificationChannel = 'submit_notification_channel'


class EventType(Enum):
    # 程序错误异常类型
    ERROR_PROGRAM = 'error_program'
    # 通知回调异常类型
    ERROR_LOGIN = 'error_login'



from agoku.pulg import IBasePlugin


class HealthCheckPluginClient(IBasePlugin):
    pass

    def on_startup(self, app):
        pass
        print(self.name, "on_startup")

    def on_shutdown(self, app):
        pass
        print("大爷的on_shutdown")

    def on_request_start(self, request):
        pass
        print("插件里面介绍", request)

    def on_request_finished(self, request,response):
        pass
        print("插件里面介绍on_request_finished", request,response.body)



import abc
from typing import Annotated

from agoku.base.context import goku_app_context
from agoku.base.gokuapp import GokuFastAPI
from fastapi import Request


class PluginException(Exception):
    """查询异常的定义"""


class IBasePlugin(metaclass=abc.ABCMeta):
    """插件基础定义"""

    name: str
    describe: str  # 描述
    app: GokuFastAPI = None
    request: Annotated[Request, goku_app_context.request] = None

    def __init__(self, name=None, describe=None, **options):
        """创建应用的插件"""
        self.name = name or self.name
        self.describe = describe

    def __repr__(self) -> str:
        """输出插件信息"""
        return f"<FastAPI.Exts.Plugin: {self.name}>"

    @property
    def installed(self):
        """检测插件是否已安装"""
        return bool(self.app)

    def setup(self, app: GokuFastAPI, name: str = None, **options):
        self.app= app
        app.on_emitter_event(event_type='on_startup')(lambda: self.on_startup(name))
        app.on_emitter_event(event_type='on_shutdown')(lambda: self.on_shutdown(name))
        app.on_emitter_event(event_type='on_request_start')(lambda req: self.on_request_start(req))
        app.on_emitter_event(event_type='on_request_finished')(lambda req, resp: self.on_request_finished(req, resp))

    @abc.abstractmethod
    def on_startup(self, name):
        pass

    @abc.abstractmethod
    def on_shutdown(self, name):
        pass

    @abc.abstractmethod
    def on_request_start(self, request):
        pass

    @abc.abstractmethod
    def on_request_finished(self, request, response):
        pass



from typing import ClassVar, Optional, Any


class Singleton:
    """一个用于实现单例设计模式的类。"""
    _instance: ClassVar[Optional[Any]] = None

    def __new__(cls, *args: Any, **kwargs: Any) -> Any:
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    @classmethod
    def _drop(cls) -> None:
        cls._instance = None



from fastapi import Request
async def get_body(request: Request):
    """
    获取客户端提交的body数据
    :param request:
    :return:
    """
    body = {}
    try:
        body = await request.json()
    except Exception as e:
        body = {}
    finally:
        request.state.body = body

    return body


async def get_body_json(request: Request):
    """
    获取客户端提交的body数据
    :param request:
    :return:
    """
    body = {}
    try:
        body = await request.json()
    except Exception as e:
        body = {}
    finally:
        request.state.body = body

    return body


#!/usr/bin/evn python
# -*- coding: utf-8 -*-

import abc

from fastapi import FastAPI
import logging


class IApplicationBuilder():
    """IApplicationBuilder接口类"""

    @classmethod
    @abc.abstractmethod
    def with_environment_settings(cls) -> 'IApplicationBuilder':
        """
        创建一个新的IApplicationBuilder实例用于设置环境参数

        Returns:
            IApplicationBuilder: IApplicationBuilder实例
        """
        raise NotImplemented

    @abc.abstractmethod
    def _instance_app(self) -> FastAPI:
        """
        创建FastAPI实例

        Returns:
            FastAPI: FastAPI实例
        """
        raise NotImplemented

    def _register_health_checks(self, app: FastAPI):
        """
        注册健康检查路由

        Args:
            app (FastAPI): FastAPI实例
        """
        pass

        @app.get("/health_checks", tags=['健康检查模块'], summary='健康检查接口')
        async def health_checks():
            return "ok"

    @abc.abstractmethod
    def _register_loguru_log_client(self, app: FastAPI) -> None:
        """
        注册自定义的日志配置插件

        Args:
            app (FastAPI): FastAPI实例
        """
        raise NotImplemented

    @abc.abstractmethod
    def _register_global_request(self, app: FastAPI) -> None:
        """
        注册全局请求

        Args:
            app (FastAPI): FastAPI实例
        """
        raise NotImplemented

    @abc.abstractmethod
    def _register_exception_handlers(self, app: FastAPI) -> None:
        """
        注册异常处理中间件

        Args:
            app (FastAPI): FastAPI实例
        """
        raise NotImplemented

    @abc.abstractmethod
    def _register_plugins(self, app: FastAPI) -> None:
        """
        注册插件

        Args:
            app (FastAPI): FastAPI实例
        """
        raise NotImplemented

    @abc.abstractmethod
    def _register_routes(self, app: FastAPI) -> None:
        """
        注册路由

        Args:
            app (FastAPI): FastAPI实例
        """
        raise NotImplemented

    @abc.abstractmethod
    def _register_middlewares(self, app: FastAPI) -> None:
        """
        注册中间件

        Args:
            app (FastAPI): FastAPI实例
        """
        raise NotImplemented

    def build(self) -> FastAPI:
        """
        构建项目

        Returns:
            FastAPI: FastAPI实例
        """
        try:
            # 约束注册流程-避免错误
            logging.critical(f'约束注册流程')
            # 创建实例对象
            app = self._instance_app()
            # 执行错误注册
            # 执行自定义的日志配置插件放在最后执行，以便获取到上下文的实例对象
            self._register_loguru_log_client(app)
            self._register_exception_handlers(app)
            # 执行插件的注册----优先于路由注册，避免部分的全局对象加载问题
            self._register_plugins(app)
            # 执行中间件的注册
            self._register_middlewares(app)
            # 注册全局请求，最外层进行注册
            self._register_global_request(app)
            # 注册路由
            self._register_routes(app)
            # 健康检查路由
            self._register_health_checks(app)
            return app
        except Exception as e:
            logging.critical(f'项目启动失败:{e}')
            raise e



from starlette.middleware import _MiddlewareClass, P
from starlette.requests import Request
import typing
from starlette.datastructures import Headers
from starlette.types import ASGIApp, Message, Receive, Scope, Send

from starlette.responses import Response

import json
from typing import Optional
from pydantic import BaseModel, Field

from agoku.base.context import goku_app_context


class ResponseInfo(BaseModel):
    headers: Optional[Headers] = Field(default=None, title="Response header")
    body: str = Field(default="", title="Response body")
    status_code: Optional[int] = Field(default=None, title="Status code")

    class Config:
        arbitrary_types_allowed = True


class GokuMiddleware:

    def __init__(self, *, app: ASGIApp, is_proxy=True):
        self.app = app
        self.is_proxy = is_proxy
        self.request: typing.Optional[Request] = None

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        if scope["type"] != "http":
            return await self.app(scope, receive, send)
        # 和上面的判断类似
        if scope["type"] not in ["http", 'websocket']:
            await self.app(scope, receive, send)
            return

        # 解决读取BODY问题
        if self.is_proxy:
            receive_ = await receive()

            async def receive():
                return receive_

        # 这里考虑直接读取一次body然后保存到对应的上下文中
        # 解析当前的请求体
        self.request = Request(scope, receive=receive)
        # 解析报文体内容
        response_info = ResponseInfo()

        # 下一个循环体
        async def _next_send(message: Message) -> None:
            if message.get("type") == "http.response.start":
                print("请求开始")
                response_info.headers = Headers(raw=message.get("headers"))
                response_info.status_code = message.get("status")
            # 解析响应体内容信息
            elif message.get("type") == "http.response.body":
                print("解析响应报文")
                if body := message.get("body"):
                    response_info.body += body.decode("utf8")

                response = Response(content=response_info.body,
                                    status_code=response_info.status_code,
                                    headers=dict(response_info.headers)
                                    )
                with goku_app_context.scope('response', response):
                    pass
                    try:
                        goku_app_context.sync_emitter.emit('on_request_finished',self.request, response)
                    except AttributeError as e:
                        self.request.app.sync_emitter.emit('on_request_finished',self.request, response)


            await send(message)

        try:
            await self.app(scope, receive, _next_send)
        finally:
            pass



from contextvars import ContextVar
import inspect

from agoku.uitls.singleton import Singleton

if hasattr(inspect, 'Parameter') and hasattr(inspect.Parameter, 'empty'):
    # 使用新版本的写法（Python 3.10+）
    empty_value = inspect.Parameter.empty
else:
    # 使用旧版本的写法
    empty_value = inspect._empty

from types import MappingProxyType
from typing import Dict, Any, Mapping, Iterator
from contextlib import contextmanager



class GokuContextRepo(Singleton):
    """一个用于表示上下文存储器的类。"""
    _global_context: Dict[str, Any]  # 全局上下文字典，存储键值对数据,类似全局变量
    _scope_context: Dict[str, ContextVar[Any]]  # 范围上下文字典，存储键值对数据，其中值为ContextVar对象

    def __init__(self) -> None:
        # 创建一个字典，键为"context"，值为self
        self._global_context = {"context": self}
        # 创建一个空字典
        self._scope_context = {}
        # 使用self._global_context创建一个MappingProxyType对象，并将其赋值给self._read_only_global_context
        # 你想要保护一个字典，防止它被修改，或者当你想要创建一个只读的字典视图时。
        # 例如，你可以使用 MappingProxyType 来包装一个字典，这样对这个字典的所有修改都会被忽略，从而保护里面的数据。
        self._read_only_global_context = MappingProxyType(self._global_context)

    def set_global(self, key: str, v: Any) -> None:
        """设置全局上下文的值。"""
        if key not in self._global_context:
            self._global_context[key] = v

    def reset_global(self, key: str) -> None:
        """重置全局上下文的值。"""
        self._global_context.pop(key, None)

    def set_local(self, key: str, value: Any) -> "ContextVar[Any].Token":
        """设置局部上下文的值。"""
        context_var = self._scope_context.get(key)
        if context_var is None:
            context_var = ContextVar(key, default=None)
            self._scope_context[key] = context_var
        return context_var.set(value)

    def reset_local(self, key: str, token: "ContextVar[Any].Token") -> None:
        """重置局部上下文的值。"""
        self._scope_context[key].reset(token)

    def get_local(self, key: str, default: Any = None) -> Any:
        """获取局部上下文的值。"""
        context_var = self._scope_context.get(key)
        return context_var.get() if context_var is not None else default

    def clear(self) -> None:
        """清空上下文存储器。"""
        self._global_context = {"context": self}
        self._scope_context.clear()

    def get(self, key: str, default: Any = None) -> Any:
        """获取全局上下文的值。,如果没有则获取局部的"""
        return self._global_context.get(key, self.get_local(key, default))

    def resolve(self, argument: str) -> Any:
        """解析复杂键。
        这个函数用于解析一个复杂键，即键中包含点号分隔的多个部分。首先根据点号分割键字符串，获取第一个部分first，
        然后通过self.get方法获取对应的值v。如果v为None，则抛出AttributeError异常。
        接着遍历剩余的部分列表keys，依次获取v的对应值。
        如果在获取值的过程中出现KeyError或AttributeError异常，则抛出AttributeError异常。最后返回解析得到的值v。
        Args:
            argument (str): 复杂键字符串。

        Returns:
            Any: 解析得到的值。

        Raises:
            AttributeError: 当self.get(first, None)返回None时，表示`self.context`不包含`first`键。
            AttributeError: 当v为None时，尝试访问v[i]或getattr(v, i)会引发KeyError或AttributeError。

        """
        first, *keys = argument.split(".")

        v = self.get(first, None)
        if v is None:
            raise AttributeError(f"`{self.context}` does not contain `{first}` key")

        for i in keys:
            try:
                v = v[i] if isinstance(v, Mapping) else getattr(v, i)
            except (KeyError, AttributeError):
                raise AttributeError(f"Attribute `{i}` not found in context")
        return v

    def resolve_context_by_name(self,
                                name: str,
                                default: Any,
                                ) -> Any:
        """动态获取上下文的值。
            #
            # contextIOOI.set_global('config', {'database': {'host': 'localhost', 'port': 3306}})
            # # 解析一个深层嵌套的键
            # db_port = contextIOOI.resolve_context_by_name(name='config.database.port',default=2)  # 输出: 3306
        """
        value: Any = empty_value

        try:
            value = self.resolve(name)

        except (KeyError, AttributeError):
            if default is not empty_value:
                value = default
                self.set_global(name, value)
        return value

    def __getattr__(self, __name: str) -> Any:
        """动态获取上下文的值。"""
        return self.get(__name)

    @property
    def context(self) -> Mapping[str, Any]:
        """获取上下文映射。
        这个函数返回一个上下文映射，它包含全局上下文和作用域上下文。全局上下文是一个只读的字典，作用域上下文是一个字典，其中的值是可调用的对象。
        函数使用字典解包来合并这两个上下文，形成最终的上下文映射。
        """
        return {**self._read_only_global_context, **{i: j.get() for i, j in self._scope_context.items()}}

    @contextmanager
    def scope(self, key: str, value: Any) -> Iterator[None]:
        """创建一个上下文作用域。"""
        token = self.set_local(key, value)
        try:
            yield
        finally:
            self.reset_local(key, token)


from typing import cast

# cast 函数在这里的作用是将 GokuContextRepo() 的实例强制转换为 GokuContextRepo 类型。
# 这样做是为了解决静态类型检查器无法推断 FastAppContextRepo() 的类型的问题。
# 使用 cast 函数可以显式地告诉类型检查器我们希望将其视为 FastAppContextRepo 类型。
goku_app_context: GokuContextRepo = cast(GokuContextRepo, GokuContextRepo())  # type: ignore[redundant-cast]

#
# contextIOOI.set_global('config', {'database': {'host': 'localhost', 'port': 3306}})
# # 解析一个深层嵌套的键
# db_port = contextIOOI.resolve_context_by_name(name='config.database.port',default=2)  # 输出: 3306
# print(db_port)
# # 使用上下文管理器在作用域内设置局部变量
# with contextIOOI.scope('key3', 'value3'):
#     scoped_value = contextIOOI.get('key3')  # 输出: 'value3'
#     print("aasaas",scoped_value)
# # 作用域结束后，key3会被自动重置
# # 设置全局上下文
# # 通过属性访问全局上下文
# # app_name = contextIOOI.app_name  # 输出: 'MyApp'
# # print("app_name",app_name.name,app_name.age)
#



"""
这个模块提供了装饰器，旨在为使用的函数提供额外的功能。
"""
import asyncio

import logging
from time import perf_counter
from typing import Any
import functools
from typing import Callable, TypeVar, Awaitable

from typing_extensions import ParamSpec

Params = ParamSpec("Params")
ReturnValue = TypeVar("ReturnValue")

logger: logging.Logger = logging.getLogger(__name__)


def with_logging(func: Callable[..., Any]) -> Callable[..., Any]:
    """
    这个装饰器在函数开始和结束执行时记录日志。
    :param func: 要装饰的函数
    :type func: Callable[..., Any]
    :return: 记录了调用信息的装饰函数
    :rtype: Callable[..., Any]
    """

    @functools.wraps(func)
    def sync_wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
        """
        添加日志功能的同步包装函数
        :param args: 传递给装饰函数的位置参数
        :type args: tuple[Any, ...]
        :param kwargs: 传递给装饰函数的关键字参数
        :type kwargs: dict[str, Any]
        :return: 装饰函数执行的结果
        :rtype: Any
        """
        logger.info("调用 %s", func.__name__)
        value = func(*args, **kwargs)
        logger.info("完成 %s", func.__name__)
        return value

    @functools.wraps(func)
    async def async_wrapper(
            *args: tuple[Any, ...], **kwargs: dict[str, Any]
    ) -> Any:
        """
        添加日志功能的异步包装函数
        :param args: 传递给装饰函数的位置参数
        :type args: tuple[Any, ...]
        :param kwargs: 传递给装饰函数的关键字参数
        :type kwargs: dict[str, Any]
        :return: 装饰函数执行的结果
        :rtype: Any
        """
        logger.info("调用 %s", func.__name__)
        value = await func(*args, **kwargs)
        logger.info("完成 %s", func.__name__)
        return value

    return async_wrapper if asyncio.iscoroutinefunction(func) else sync_wrapper


def benchmark(func: Callable[..., Any]) -> Callable[..., Any]:
    """
    通过记录装饰函数的执行时间，提供基准测试的功能
    :param func: 要执行的函数
    :type func: Callable[..., Any]
    :return: 添加了基准测试功能的装饰函数
    :rtype: Callable[..., Any]
    """

    @functools.wraps(func)
    def sync_wrapper(*args: tuple[Any, ...], **kwargs: dict[str, Any]) -> Any:
        """
        添加基准测试功能的同步包装函数
        :param args: 传递给装饰函数的位置参数
        :type args: tuple[Any, ...]
        :param kwargs: 传递给装饰函数的关键字参数
        :type kwargs: dict[str, Any]
        :return: 装饰函数执行的结果
        :rtype: Any
        """
        start_time: float = perf_counter()
        value = func(*args, **kwargs)
        end_time: float = perf_counter()
        run_time: float = end_time - start_time
        logger.info("%s的执行需要 %s 秒。", func.__name__, run_time)
        return value

    @functools.wraps(func)
    async def async_wrapper(
            *args: tuple[Any, ...], **kwargs: dict[str, Any]
    ) -> Any:
        """
        添加基准测试功能的异步包装函数
        :param args: 传递给装饰函数的位置参数
        :type args: tuple[Any, ...]
        :param kwargs: 传递给装饰函数的关键字参数
        :type kwargs: dict[str, Any]
        :return: 装饰函数执行的结果
        :rtype: Any
        """
        start_time: float = perf_counter()
        value = await func(*args, **kwargs)
        end_time: float = perf_counter()
        run_time: float = end_time - start_time
        logger.info("%s的执行需要 %s 秒。", func.__name__, run_time)
        return value

    return async_wrapper if asyncio.iscoroutinefunction(func) else sync_wrapper


def fun_hooks(
        f: Callable[Params, Awaitable[ReturnValue]],
) -> Callable[Params, Awaitable[ReturnValue]]:
    @functools.wraps(f)
    def wrapper(*args: Params.args, **kwargs: Params.kwargs) -> ReturnValue:
        r = f(*args, **kwargs)
        return r

    return wrapper



from setuptools import setup, find_packages

setup(
    name='agokufastapi',
    version='1.0.0',
    author='zyx',
    author_email='308711822@qq.com',
    description='基于Fatapi框架的封装扩展库',
    long_description='个人封装的库',
    long_description_content_type='text/markdown',
    url='',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.11',
    ],
    python_requires='>=3.7',
    install_requires=[
        'aiohttp==3.9.3',
        'aiojobs==1.2.1',
        'fastapi==0.109.2',
        'pydantic==2.6.1',
        'pyee==11.1.0',
        'setuptools==65.5.0',
        'starlette==0.26.1',
        'typing_extensions==4.9.0',
        'uvicorn==0.27.1',
    ],
)



from typing import Annotated

from fastapi import Depends, Request, BackgroundTasks
from agoku.base.context import goku_app_context
from agoku.uitls.body_parse import get_body, get_body_json


class GlobalContextRequest:
    """
    全局请求上下文类
    curr_global_dependencies = [Depends(GlobalContextRequest())]

    Attributes:
        dependencie_name (str): 依赖名称
    """

    def __init__(self, *, dependencie_name: str = 'dependencie_name'):
        self.dependencie_name = dependencie_name

    async def __call__(self, request: Request, background_tasks: BackgroundTasks, bodys=Depends(get_body_json)):
        """
        请求上下文管理器
        Args:
            request (Request): 请求对象

        Returns:
            None
        """
        pass
        # 触发请求开始事件
        # goku_app_context.async_emitter.emit('on_request_start', request)
        goku_app_context.sync_emitter.emit('on_request_start', request)
        # 设置请求上下文
        with goku_app_context.scope('request', request):
            with goku_app_context.scope('background_tasks', background_tasks):
                request.state.current_background_tasks = background_tasks
                try:
                    yield
                finally:
                    # 清空当前的请求
                    pass
                # print('goku_app_context',goku_app_context.response)



import asyncio
import inspect
from threading import Thread
import logging


# 异步编程相关模块
class Worker(Thread):
    """
    工作线程类

    Args:
        name (str, optional): 工作线程的名称，默认为 "Worker"。 Defaults to "Worker".

    # 使用示例
    from goku.libs.worker import Worker
    class AlertsGather1115555(Worker):
        async def work(self):
            while True:
                print("AlertsGather1115555循环一直输出")

    class AlertsGather111(Worker):
       async def work(self):
            while True:
                pass
                print("AlertsGather111循环一直输出")

    if __name__ == '__main__':
        AlertsGather111().begin()
        AlertsGather1115555.begin()
    """

    def __init__(self, name: str = "Worker", *args, **kwargs) -> None:
        """
        工作线程的初始化方法

        Args:
            name (str, optional): 工作线程的名称，默认为 "Worker"。 Defaults to "Worker".
            *args: 可变参数列表。
            **kwargs: 可变关键字参数列表。
        """
        super().__init__(name=name, *args, **kwargs)
        self.logger = logging.getLogger("Worker")


    def run(self):
        # 启动线程时执行的方法
        self.logger.info(f"Starting worker {self.name}")
        # 使用inspect.iscoroutinefunction(self.work)检查self.work是否定义为异步函数。
        # 如果self.work是异步函数，那么：
        # 创建一个新的 asyncio 事件循环（loop = asyncio.new_event_loop()）。
        # 设置这个新创建的事件循环为当前活动的事件循环（asyncio.set_event_loop(loop)）。
        # 在日志中记录“Asynchronous”，表示即将执行异步操作。
        # 使用asyncio.Task(self.work())将异步函数封装为一个任务，并通过add_done_callback添加回调函数，当任务完成后调用loop.shutdown_asyncgens()清理生成器。
        # 调用loop.run_until_complete(task)来运行并等待此异步任务完成。
        # 完成任务后关闭事件循环（loop.close()）。
        # 如果self.work不是异步函数（即同步函数），直接调用并执行self.work()。
        try:
            # 检查工作方法是否为异步函数
            if inspect.iscoroutinefunction(self.work):
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
                self.logger.info("Asynchronous")
                # 运行异步函数，添加了异常处理
                task = asyncio.Task(self.work())
                task.add_done_callback(lambda t: loop.shutdown_asyncgens())
                loop.run_until_complete(task)
                loop.close()
            else:
                # 运行同步函数
                self.work()
        except Exception as e:
            self.logger.error(f"An error occurred in worker {self.name}: {e}")

    @classmethod
    def begin(cls, name: str = "Worker", *args, **kwargs) -> 'Worker':
        # 创建并返回一个 Worker 实例
        instance = cls(name=name, *args, **kwargs)
        instance.start()
        # instance.join()
        return instance

    def work(self) -> None:
        # 占位符
        pass

if __name__ == '__main__':
    class AlertsGather1115555(Worker):
        async def work(self):
            while True:
                print("AlertsGather1115555循环一直输出")


    class AlertsGather111(Worker):
        async def work(self):
            while True:
                pass
                print("AlertsGather111循环一直输出")


    if __name__ == '__main__':
        AlertsGather111().begin()
        AlertsGather1115555.begin()


import asyncio
import typing
from contextlib import asynccontextmanager
from typing import Callable, Annotated

import aiojobs
from fastapi import FastAPI
from fastapi.types import DecoratedCallable
from pyee import EventEmitter, AsyncIOEventEmitter

from agoku.base.context import goku_app_context
from agoku.base.dependencys import GlobalContextRequest
from fastapi import Depends

from agoku.base.middleware import GokuMiddleware



class GokuFastAPI(FastAPI):
    def __init__(self, *args, **kwargs):

        # 创建事件发射器--用户同步执行逻辑的时候的事件处理
        self.sync_emitter = EventEmitter()
        # 创建异步事件发射器--可用户类似后台异步任务的处理，单仅限于在异步事件循环中执行相关发布任务
        self.async_emitter = AsyncIOEventEmitter()
        self.async_background_aiojobs = None

        # 设置全局的依赖注入项
        curr_global_dependencies = [Depends(GlobalContextRequest())]
        if kwargs.get('dependencies'):
            curr_global_dependencies += kwargs.get('dependencies')
            kwargs.pop('dependencies')
        # 设置生命周期回调
        curr_lifespan = self.lifespan
        super().__init__(*args, **kwargs, lifespan=curr_lifespan, dependencies=curr_global_dependencies)
        # self.add_middleware(GokuMiddleware, is_proxy=True)
        self.add_middleware(GokuMiddleware,is_proxy=True)

    def init_global_goku_app_context(self,app: "GokuFastAPI"):
        # 设置全局的依赖注入项-sync_emitter
        goku_app_context.set_global('sync_emitter', self.sync_emitter)
        #  设置全局的依赖注入项- async_emitter
        goku_app_context.set_global('async_emitter', self.async_emitter)
        goku_app_context.set_global('async_background_aiojobs', self.async_background_aiojobs)

    @asynccontextmanager
    async def lifespan(self,app: "GokuFastAPI"):
        # 启动事件
        app.emit_startup_server()
        # 创建异步任务调度器--用于异步任务调度
        app.async_background_aiojobs = aiojobs.Scheduler()
        # 创建异步任务调度器--用于异步任务调度
        app.init_global_goku_app_context(app)
        #  设置全局的依赖注入项- async_background_aiojobs
        goku_app_context.set_global('async_background_aiojobs', app.async_background_aiojobs)
        # 生命周回调
        yield
        # 关闭事件
        app.emit_shutdown_server()

    def add_plugin(self, plugin_class, name, *args, **kwargs) -> None:
        # 创建插件实例，并使用提供的参数
        plugin_instance = plugin_class(name=name, *args, **kwargs)
        # 调用插件实例的 setup 方法，传入当前应用实例（self）
        plugin_instance.setup(app=self)


    def on_emitter_event(
            self, event_type: Annotated[str, ...] = None,
    ) -> Callable[[DecoratedCallable], DecoratedCallable]:
        def decorator(func: DecoratedCallable) -> DecoratedCallable:
            # 判断函数是否是异步函数
            if asyncio.iscoroutinefunction(func):
                # 若是异步函数，则使用async_emitter对象的on方法注册事件和函数
                self.async_emitter.on(event_type if event_type else func.__name__, func)
            else:
                # 若不是异步函数，则使用sync_emitter对象的on方法注册事件和函数
                self.sync_emitter.on(event_type if event_type else func.__name__, func)
            return func

        return decorator

    def emit_request_start_sync(self, request):
        """
        同步地发出请求开始事件。

        参数：
        - self：当前对象的引用
        - request：请求对象
        """
        self.sync_emitter.emit('on_request_start', request)

    def emit_request_finished_sync(self, request, response):
        """
        同步地发出请求完成事件。

        参数：
        - self：当前对象的引用
        - request：请求对象
        - response：响应对象
        """
        self.sync_emitter.emit('on_request_finished', request, response)

    def emit_startup_server(self):
        """
        同步地发出请求开始事件。

        参数：
        - self：当前对象的引用
        - request：请求对象
        """
        self.sync_emitter.emit('on_startup', self)

    def emit_shutdown_server(self):
        """
        同步地发出请求开始事件。

        参数：
        - self：当前对象的引用
        - request：请求对象
        """
        self.sync_emitter.emit('on_shutdown', self)



